
#pragma once
#include <stdio.h>
#include <new>
#include "MyException.h"



template <class T>
class Stack
{
 private:
   T *mem, *top;
   int  maxsize;
 public:
   Stack(int msize);
   Stack(const Stack<T> & st);
   ~Stack() { delete [] mem; }

   bool Push(const T & x);
   bool Pop(T & x);
   bool Del();

   T & Top();
   const T & Top() const;

   int Size() const { return top-mem+1; }
   int Maxsize() const { return maxsize; }

   friend void PrintStacks();

   const Stack & operator=(const Stack &st);
   bool operator!=(const Stack &st) const;

};

template <class T>
Stack<T>::Stack(int msize)
{
 try {
   maxsize = msize; 
   mem = new T[maxsize];
   top = mem - 1;
 } catch (std::bad_alloc) {
    throw new MyException("Stack: cannot allocate memory\n", EC_MEMORY);
 }
}

template <class T>
Stack<T>::Stack(const Stack<T> & st)
{
 try {
   maxsize = st.maxsize;
   mem = new  T[maxsize];
   top = mem - 1;
   int size = st.Size();
   for (int i=0; i<size; i++) { mem[i] = st.mem[i]; } 
   top = mem + size - 1;
 } catch (std::bad_alloc) {
    throw new MyException("Stack: cannot apply copy constructor\n", EC_MEMORY);
 }
}


template <class T>
bool Stack<T>::Push(const T & x)
{
   if (Size() == maxsize) return false; 
   *(++top) = x;
   return true;
}

template <class T>
bool Stack<T>::Pop(T & x)
{
   if (Size() == 0) return false; 
   x = *top--;  
   return true;
}

template <class T>
bool Stack<T>::Del()
{
   if (Size() == 0) return false; 
   top--;  
   return true;
}

template <class T>
T & Stack<T>::Top()
{
   if (Size() == 0) throw new MyException("Stack::Top: empty stack\n", EC_ACCESS); 
   return *top;
}

template <class T>
const Stack<T> & Stack<T>::operator=(const Stack<T> &st)
{
 try {
   delete [] mem;
   maxsize = st.maxsize;
   mem = new  T[maxsize];
   top = mem - 1;
   int size = st.Size();
   for (int i=0; i<size; i++) { mem[i] = st.mem[i]; } 
   top = mem + size - 1;
   return *this;
 } catch (std::bad_alloc) {
    throw new MyException("Stack: cannot apply copy constructor\n", EC_MEMORY);
 }
}


template <class T>
bool Stack<T>::operator!=(const Stack<T> &st) const
{
   if (Size()!=st.Size()) return true;
   for (int i=0; i<Size(); i++) { if( mem[i] != st.mem[i]) return true; } 
   return false;
}












